require 'tennpipes-base/tasks'
require 'tennpipes-init/command'
require 'active_support/ordered_hash'

module Tennpipes
  ##
  # This module it's used for register generators.
  #
  # Can be useful for 3rd party generators:
  #
  #   # custom_generator.rb
  #   class CustomGenerator < Thor::Group
  #     Tennpipes::Generators.add_generator(:custom_generator, self)
  #   end
  #
  # Now for handle generators in tennpipes you need to add it to into +load_paths+.
  #
  # Tennpipes::Generators.load_paths << "custom_generator.rb"
  #
  module Generators
    # Defines the absolute path to the tennpipes source folder.
    DEV_PATH = File.expand_path("../../", File.dirname(__FILE__))

    class << self
      ##
      # Store our generators paths.
      #
      def load_paths
        @_files ||= []
      end

      ##
      # Return an ordered list of task with their class.
      #
      def mappings
        @_mappings ||= ActiveSupport::OrderedHash.new
      end

      ##
      # Global add a new generator class to +tennpipes-init+.
      #
      # @param [Symbol] name
      #   Key name for generator mapping.
      # @param [Class] klass
      #   Class of generator.
      #
      # @return [Hash] generator mappings
      #
      # @example
      #   Tennpipes::Generators.add_generator(:controller, Controller)
      #
      def add_generator(name, klass)
        mappings[name] = klass
      end

      ##
      # Load Global Actions and Component Actions then all files in +load_path+.
      #
      def load_components!
        require 'tennpipes-init/generators/actions'
        require 'tennpipes-init/generators/components/actions'
        require 'tennpipes-init/generators/runner'
        load_paths.flatten.each { |file| require file  }
      end
    end
  end
end

# Add our generators to Tennpipes::Generators.
Tennpipes::Generators.load_paths << Dir[File.dirname(__FILE__) + '/tennpipes-init/generators/{project,app,mailer,controller,helper,model,migration,plugin,component,task}.rb']

# Add our tasks to tennpipes-base.
Tennpipes::Tasks.files << Dir[File.dirname(__FILE__) + "/tennpipes-init/tennpipes-tasks/**/*.rb"]
